@@ -6,23 +6,23 @@ package main
 
 import (
 	"context"
-	"expvar"
 	"flag"
 	"fmt"
-	"io/ioutil"
 	"math/rand"
 	"net"
 	"os"
 	"os/signal"
+	"path"
 	"path/filepath"
-	"strconv"
+	"strings"
 	"syscall"
 	"time"
 
 	"blitiri.com.ar/go/chasquid/internal/config"
 	"blitiri.com.ar/go/chasquid/internal/courier"
+	"blitiri.com.ar/go/chasquid/internal/domaininfo"
 	"blitiri.com.ar/go/chasquid/internal/dovecot"
-	"blitiri.com.ar/go/chasquid/internal/expvarom"
+	"blitiri.com.ar/go/chasquid/internal/localrpc"
 	"blitiri.com.ar/go/chasquid/internal/maillog"
 	"blitiri.com.ar/go/chasquid/internal/normalize"
 	"blitiri.com.ar/go/chasquid/internal/smtpsrv"
@@ -41,22 +41,6 @@ var (
 	showVer = flag.Bool("version", false, "show version and exit")
 )
 
-// Build information, overridden at build time using
-// -ldflags="-X main.version=blah".
-var (
-	version      = "undefined"
-	sourceDateTs = "0"
-)
-
-var (
-	versionVar = expvar.NewString("chasquid/version")
-
-	sourceDate      time.Time
-	sourceDateVar   = expvar.NewString("chasquid/sourceDateStr")
-	sourceDateTsVar = expvarom.NewInt("chasquid/sourceDateTimestamp",
-		"timestamp when the binary was built, in seconds since epoch")
-)
-
 func main() {
 	flag.Parse()
 	log.Init()
@@ -90,8 +74,6 @@ func main() {
 
 	initMailLog(conf.MailLogPath)
 
-	go signalHandler()
-
 	if conf.MonitoringAddress != "" {
 		go launchMonitoringServer(conf)
 	}
@@ -102,7 +84,7 @@ func main() {
 	s.HookPath = "hooks/"
 	s.HAProxyEnabled = conf.HaproxyIncoming
 
-	s.SetAliasesConfig(conf.SuffixSeparators, conf.DropCharacters)
+	s.SetAliasesConfig(*conf.SuffixSeparators, *conf.DropCharacters)
 
 	if conf.DovecotAuth {
 		loadDovecot(s, conf.DovecotUserdbPath, conf.DovecotClientPath)
@@ -110,39 +92,29 @@ func main() {
 
 	// Load certificates from "certs/<directory>/{fullchain,privkey}.pem".
 	// The structure matches letsencrypt's, to make it easier for that case.
-	log.Infof("Loading certificates")
+	log.Infof("Loading certificates:")
 	for _, info := range mustReadDir("certs/") {
+		if info.Type().IsRegular() {
+			// Ignore regular files, we only care about directories.
+			continue
+		}
+
 		name := info.Name()
 		dir := filepath.Join("certs/", name)
-		if fi, err := os.Stat(dir); err == nil && !fi.IsDir() {
-			// Skip non-directories.
-			continue
-		}
-
-		log.Infof("  %s", name)
-
-		certPath := filepath.Join(dir, "fullchain.pem")
-		if _, err := os.Stat(certPath); os.IsNotExist(err) {
-			continue
-		}
-		keyPath := filepath.Join(dir, "privkey.pem")
-		if _, err := os.Stat(keyPath); os.IsNotExist(err) {
-			continue
-		}
-
-		err := s.AddCerts(certPath, keyPath)
-		if err != nil {
-			log.Fatalf("    %v", err)
-		}
+		loadCert(name, dir, s)
 	}
 
 	// Load domains from "domains/".
-	log.Infof("Domain config paths:")
+	log.Infof("Loading domains:")
 	for _, info := range mustReadDir("domains/") {
 		domain, err := normalize.Domain(info.Name())
 		if err != nil {
 			log.Fatalf("Invalid name %+q: %v", info.Name(), err)
 		}
+		if info.Type().IsRegular() {
+			// Ignore regular files, we only care about directories.
+			continue
+		}
 		dir := filepath.Join("domains", info.Name())
 		loadDomain(domain, dir, s)
 	}
@@ -152,7 +124,11 @@ func main() {
 	// as a remote domain (for loops, alias resolutions, etc.).
 	s.AddDomain("localhost")
 
-	dinfo := s.InitDomainInfo(conf.DataDir + "/domaininfo")
+	dinfo, err := domaininfo.New(conf.DataDir + "/domaininfo")
+	if err != nil {
+		log.Fatalf("Error opening domain info database: %v", err)
+	}
+	s.SetDomainInfo(dinfo)
 
 	stsCache, err := sts.NewCache(conf.DataDir + "/sts-cache")
 	if err != nil {
@@ -189,14 +165,23 @@ func main() {
 		log.Fatalf("No address to listen on")
 	}
 
+	go localrpc.DefaultServer.ListenAndServe(conf.DataDir + "/localrpc-v1")
+
+	go signalHandler(dinfo, s)
+
 	s.ListenAndServe()
 }
 
 func loadAddresses(srv *smtpsrv.Server, addrs []string, ls []net.Listener, mode smtpsrv.SocketMode) int {
 	naddr := 0
 	for _, addr := range addrs {
-		// The "systemd" address indicates we get listeners via systemd.
-		if addr == "systemd" {
+		if addr == "" {
+			// An empty address is invalid, to prevent accidental
+			// misconfiguration.
+			log.Errorf("Invalid empty listening address for %v", mode)
+			log.Fatalf("If you want to disable %v, remove it from the config", mode)
+		} else if addr == "systemd" {
+			// The "systemd" address indicates we get listeners via systemd.
 			srv.AddListeners(ls, mode)
 			naddr += len(ls)
 		} else {
@@ -232,15 +217,17 @@ func initMailLog(path string) {
 	}
 }
 
-func signalHandler() {
+func signalHandler(dinfo *domaininfo.DB, srv *smtpsrv.Server) {
 	var err error
 
 	signals := make(chan os.Signal, 1)
-	signal.Notify(signals, syscall.SIGHUP)
+	signal.Notify(signals, syscall.SIGHUP, syscall.SIGTERM, syscall.SIGINT)
 
 	for {
 		switch sig := <-signals; sig {
 		case syscall.SIGHUP:
+			log.Infof("Received SIGHUP, reloading")
+
 			// SIGHUP triggers a reopen of the log files. This is used for log
 			// rotation.
 			err = log.Default.Reopen()
@@ -252,31 +239,73 @@ func signalHandler() {
 			if err != nil {
 				log.Fatalf("Error reopening maillog: %v", err)
 			}
+
+			// We don't want to reload the domain info database periodically,
+			// as it can be expensive, and it is not expected that the user
+			// changes this behind chasquid's back.
+			err = dinfo.Reload()
+			if err != nil {
+				log.Fatalf("Error reloading domain info: %v", err)
+			}
+
+			// Also trigger a server reload.
+			srv.Reload()
+		case syscall.SIGTERM, syscall.SIGINT:
+			log.Fatalf("Got signal to exit: %v", sig)
 		default:
 			log.Errorf("Unexpected signal %v", sig)
 		}
 	}
 }
 
+// Helper to load a single certificate configuration into the server.
+func loadCert(name, dir string, s *smtpsrv.Server) {
+	log.Infof("  %s", name)
+
+	// Ignore directories that don't have both keys.
+	// We warn about this because it can be hard to debug otherwise.
+	certPath := filepath.Join(dir, "fullchain.pem")
+	if _, err := os.Stat(certPath); err != nil {
+		log.Infof("    skipping: %v", err)
+		return
+	}
+	keyPath := filepath.Join(dir, "privkey.pem")
+	if _, err := os.Stat(keyPath); err != nil {
+		log.Infof("    skipping: %v", err)
+		return
+	}
+
+	err := s.AddCerts(certPath, keyPath)
+	if err != nil {
+		log.Fatalf("    %v", err)
+	}
+}
+
 // Helper to load a single domain configuration into the server.
 func loadDomain(name, dir string, s *smtpsrv.Server) {
 	log.Infof("  %s", name)
 	s.AddDomain(name)
 
-	if _, err := os.Stat(dir + "/users"); err == nil {
-		log.Infof("    adding users")
-		udb, err := userdb.Load(dir + "/users")
-		if err != nil {
-			log.Errorf("      error: %v", err)
-		} else {
-			s.AddUserDB(name, udb)
-		}
+	udb, err := userdb.Load(dir + "/users")
+	if os.IsNotExist(err) {
+		// No users file present, that's okay.
+	} else if err != nil {
+		log.Errorf("    users file error: %v", err)
+	} else {
+		s.AddUserDB(name, udb)
 	}
 
-	log.Infof("    adding aliases")
-	err := s.AddAliasesFile(name, dir+"/aliases")
+	err = s.AddAliasesFile(name, dir+"/aliases")
 	if err != nil {
-		log.Errorf("      error: %v", err)
+		log.Errorf("    aliases file error: %v", err)
+	}
+
+	err = loadDKIM(name, dir, s)
+	if err != nil {
+		// DKIM errors are fatal because if the user set DKIM up, then we
+		// don't want it to be failing silently, as that could cause
+		// deliverability issues.
+		log.Fatalf("    DKIM loading error: %v", err)
 	}
 }
 
@@ -290,9 +319,29 @@ func loadDovecot(s *smtpsrv.Server, userdb, client string) {
 	}
 }
 
+func loadDKIM(domain, dir string, s *smtpsrv.Server) error {
+	glob := path.Clean(dir + "/dkim:*.pem")
+	pems, err := filepath.Glob(glob)
+	if err != nil {
+		return err
+	}
+
+	for _, pem := range pems {
+		base := filepath.Base(pem)
+		selector := strings.TrimPrefix(base, "dkim:")
+		selector = strings.TrimSuffix(selector, ".pem")
+
+		err = s.AddDKIMSigner(domain, selector, pem)
+		if err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 // Read a directory, which must have at least some entries.
-func mustReadDir(path string) []os.FileInfo {
-	dirs, err := ioutil.ReadDir(path)
+func mustReadDir(path string) []os.DirEntry {
+	dirs, err := os.ReadDir(path)
 	if err != nil {
 		log.Fatalf("Error reading %q directory: %v", path, err)
 	}
@@ -302,16 +351,3 @@ func mustReadDir(path string) []os.FileInfo {
 
 	return dirs
 }
-
-func parseVersionInfo() {
-	versionVar.Set(version)
-
-	sdts, err := strconv.ParseInt(sourceDateTs, 10, 0)
-	if err != nil {
-		panic(err)
-	}
-
-	sourceDate = time.Unix(sdts, 0)
-	sourceDateVar.Set(sourceDate.Format("2006-01-02 15:04:05 -0700"))
-	sourceDateTsVar.Set(sdts)
-}
