@@ -1,24 +1,20 @@
 // chasquid-util is a command-line utility for chasquid-related operations.
-//
-// Don't include it in the coverage build.
-// +build !coverage
 
 package main
 
 import (
 	"bytes"
 	"fmt"
-	"io/ioutil"
-	"net/url"
 	"os"
 	"path/filepath"
+	"sort"
 	"strconv"
 	"strings"
 	"syscall"
 
-	"blitiri.com.ar/go/chasquid/internal/aliases"
 	"blitiri.com.ar/go/chasquid/internal/config"
 	"blitiri.com.ar/go/chasquid/internal/envelope"
+	"blitiri.com.ar/go/chasquid/internal/localrpc"
 	"blitiri.com.ar/go/chasquid/internal/normalize"
 	"blitiri.com.ar/go/chasquid/internal/userdb"
 	"golang.org/x/term"
@@ -28,17 +24,30 @@ import (
 // Usage to show users on --help or invocation errors.
 const usage = `
 Usage:
-  chasquid-util [options] user-add <user@domain> [--password=<password>]
+  chasquid-util [options] user-add <user@domain> [--password=<password>] [--receive_only]
+    Add a user to the userdb.
   chasquid-util [options] user-remove <user@domain>
+    Remove a user from the userdb.
   chasquid-util [options] authenticate <user@domain> [--password=<password>]
+    Authenticate a user.
   chasquid-util [options] check-userdb <domain>
+    Check if the userdb for the given domain is accessible.
   chasquid-util [options] aliases-resolve <address>
+    Resolve an address. Talks to the running chasquid.
   chasquid-util [options] domaininfo-remove <domain>
+    Remove domaininfo for the given domain. Talks to the running chasquid.
   chasquid-util [options] print-config
-  chasquid-util [options] aliases-add <source> <target>
+    Print the current chasquid configuration.
+
+  chasquid-util [options] dkim-keygen <domain> [<selector> <private-key.pem>] [--algo=rsa3072|rsa4096|ed25519]
+    Generate a new DKIM key pair for the domain.
+  chasquid-util [options] dkim-dns <domain> [<selector> <private-key.pem>]
+    Print the DNS TXT record to use for the domain, selector and
+    private key.
 
 Options:
   -C=<path>, --configdir=<path>  Configuration directory
+  -v                             Verbose mode
 `
 
 // Command-line arguments.
@@ -78,7 +87,13 @@ func main() {
 		"aliases-resolve":   aliasesResolve,
 		"print-config":      printConfig,
 		"domaininfo-remove": domaininfoRemove,
-		"aliases-add":       aliasesAdd,
+		"dkim-keygen":       dkimKeygen,
+		"dkim-dns":          dkimDNS,
+
+		// These exist for testing purposes and may be removed in the future.
+		// Do not rely on them.
+		"dkim-verify": dkimVerify,
+		"dkim-sign":   dkimSign,
 	}
 
 	cmd := args["$1"]
@@ -90,9 +105,10 @@ func main() {
 	}
 }
 
-// Fatalf prints the given message, then exits the program with an error code.
+// Fatalf prints the given message to stderr, then exits the program with an
+// error code.
 func Fatalf(s string, arg ...interface{}) {
-	fmt.Printf(s+"\n", arg...)
+	fmt.Fprintf(os.Stderr, s+"\n", arg...)
 	os.Exit(1)
 }
 
@@ -141,12 +157,25 @@ func checkUserDB() {
 	fmt.Println("Database loaded")
 }
 
-// chasquid-util user-add <user@domain> [--password=<password>]
+// chasquid-util user-add <user@domain> [--password=<password>] [--receive_only]
 func userAdd() {
 	user, _, db := userDBFromArgs(true)
-	password := getPassword()
 
-	err := db.AddUser(user, password)
+	_, recvOnly := args["--receive_only"]
+	_, hasPassword := args["--password"]
+
+	if recvOnly && hasPassword {
+		Fatalf("Cannot specify both --receive_only and --password")
+	}
+
+	var err error
+	if recvOnly {
+		err = db.AddDeniedUser(user)
+	} else {
+		password := getPassword()
+		err = db.AddUser(user, password)
+	}
+
 	if err != nil {
 		Fatalf("Error adding user: %v", err)
 	}
@@ -222,42 +251,26 @@ func aliasesResolve() {
 	if err != nil {
 		Fatalf("Error loading config: %v", err)
 	}
-	_ = os.Chdir(configDir)
 
-	r := aliases.NewResolver()
-	r.SuffixSep = *conf.SuffixSeparators
-	r.DropChars = *conf.DropCharacters
-
-	domainDirs, err := ioutil.ReadDir("domains/")
-	if err != nil {
-		Fatalf("Error reading domains/ directory: %v", err)
-	}
-	if len(domainDirs) == 0 {
-		Fatalf("No domains found in config")
-	}
-
-	for _, info := range domainDirs {
-		name := info.Name()
-		aliasfile := "domains/" + name + "/aliases"
-		r.AddDomain(name)
-		err := r.AddAliasesFile(name, aliasfile)
-		if err == nil {
-			fmt.Printf("%s: loaded %q\n", name, aliasfile)
-		} else if err != nil && os.IsNotExist(err) {
-			fmt.Printf("%s: no aliases file\n", name)
-		} else {
-			fmt.Printf("%s: error loading %q: %v\n", name, aliasfile, err)
-		}
-	}
-
-	rcpts, err := r.Resolve(args["$2"])
+	c := localrpc.NewClient(conf.DataDir + "/localrpc-v1")
+	vs, err := c.Call("AliasResolve", "Address", args["$2"])
 	if err != nil {
 		Fatalf("Error resolving: %v", err)
 	}
-	for _, rcpt := range rcpts {
-		fmt.Printf("%v  %s\n", rcpt.Type, rcpt.Addr)
-	}
 
+	// Result is a map of type -> []addresses.
+	// Sort the types for deterministic output.
+	ts := []string{}
+	for t := range vs {
+		ts = append(ts, t)
+	}
+	sort.Strings(ts)
+
+	for _, t := range ts {
+		for _, a := range vs[t] {
+			fmt.Printf("%v  %s\n", t, a)
+		}
+	}
 }
 
 // chasquid-util print-config
@@ -272,77 +285,18 @@ func printConfig() {
 
 // chasquid-util domaininfo-remove <domain>
 func domaininfoRemove() {
-	domain := args["$2"]
-
 	conf, err := config.Load(configDir+"/chasquid.conf", "")
 	if err != nil {
 		Fatalf("Error loading config: %v", err)
 	}
 
-	// File for the corresponding domain.
-	// Note this is making some assumptions about the data layout and
-	// protoio's storage structure, so it will need adjustment if they change.
-	file := conf.DataDir + "/domaininfo/s:" + url.QueryEscape(domain)
-	err = os.Remove(file)
+	c := localrpc.NewClient(conf.DataDir + "/localrpc-v1")
+	_, err = c.Call("DomaininfoClear", "Domain", args["$2"])
 	if err != nil {
-		Fatalf("Error removing file: %v", err)
+		Fatalf("Error removing domaininfo entry: %v", err)
 	}
 }
 
-// chasquid-util aliases-add <source> <target>
-func aliasesAdd() {
-	source := args["$2"]
-	target := args["$3"]
-
-	user, domain := envelope.Split(source)
-	if domain == "" {
-		Fatalf("Domain required in source address")
-	}
-
-	if target == "" {
-		Fatalf("Target must be present")
-	}
-
-	// Ensure the domain exists.
-	if _, err := os.Stat(filepath.Join(configDir, "domains", domain)); os.IsNotExist(err) {
-		Fatalf("Domain doesn't exist")
-	}
-
-	conf, err := config.Load(configDir+"/chasquid.conf", "")
-	if err != nil {
-		Fatalf("Error loading config: %v", err)
-	}
-	_ = os.Chdir(configDir)
-
-	// Setup alias resolver.
-	r := aliases.NewResolver()
-	r.SuffixSep = *conf.SuffixSeparators
-	r.DropChars = *conf.DropCharacters
-
-	r.AddDomain(domain)
-	aliasesFilePath := filepath.Join("domains", domain, "aliases")
-	if err := r.AddAliasesFile(domain, aliasesFilePath); err != nil {
-		Fatalf("%s: error loading %q: %v", domain, aliasesFilePath, err)
-	}
-
-	// Check for existing entry.
-	if _, ok := r.Exists(source); ok {
-		Fatalf("There's already an entry for %v", source)
-	}
-
-	// Append the new entry.
-	aliasesFile, err := os.OpenFile(aliasesFilePath, os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		Fatalf("Couldn't open %s: %v", aliasesFilePath, err)
-	}
-	_, err = fmt.Fprintf(aliasesFile, "%s: %s\n", user, target)
-	if err != nil {
-		Fatalf("Couldn't write to %s: %v", aliasesFilePath, err)
-	}
-	aliasesFile.Close()
-	fmt.Println("Added alias")
-}
-
 // parseArgs parses the command line arguments, and returns a map.
 //
 // Arguments starting with "-" will be parsed as key-value pairs, and
