@@ -1,7 +1,6 @@
 // Package userdb implements a simple user database.
 //
-//
-// Format
+// # Format
 //
 // The user database is a file containing a list of users and their passwords,
 // encrypted with some scheme.
@@ -13,21 +12,18 @@
 // Users must be UTF-8 and NOT contain whitespace; the library will enforce
 // this.
 //
-//
-// Schemes
+// # Schemes
 //
 // The default scheme is SCRYPT, with hard-coded parameters. The API does not
 // allow the user to change this, at least for now.
 // A PLAIN scheme is also supported for debugging purposes.
 //
-//
-// Writing
+// # Writing
 //
 // The functions that write a database file will not preserve ordering,
 // invalid lines, empty lines, or any formatting.
 //
 // It is also not safe for concurrent use from different processes.
-//
 package userdb
 
 //go:generate protoc --go_out=. --go_opt=paths=source_relative userdb.proto
@@ -127,6 +123,8 @@ func (p *Password) PasswordMatches(plain string) bool {
 		return s.Scrypt.PasswordMatches(plain)
 	case *Password_Plain:
 		return s.Plain.PasswordMatches(plain)
+	case *Password_Denied:
+		return false
 	default:
 		return false
 	}
@@ -168,6 +166,22 @@ func (db *DB) AddUser(name, plainPassword string) error {
 	return nil
 }
 
+// AddDenied to the database. If the user is already present, override it.
+// Note we enforce that the name has been normalized previously.
+func (db *DB) AddDeniedUser(name string) error {
+	if norm, err := normalize.User(name); err != nil || name != norm {
+		return errors.New("invalid username")
+	}
+
+	db.mu.Lock()
+	db.db.Users[name] = &Password{
+		Scheme: &Password_Denied{&Denied{}},
+	}
+	db.mu.Unlock()
+
+	return nil
+}
+
 // RemoveUser from the database. Returns True if the user was there, False
 // otherwise.
 func (db *DB) RemoveUser(name string) bool {
