@@ -2,16 +2,20 @@ package main
 
 import (
 	"context"
+	"expvar"
 	"flag"
 	"fmt"
 	"html/template"
 	"net/http"
 	"os"
 	"runtime"
+	"runtime/debug"
+	"strconv"
 	"time"
 
 	"blitiri.com.ar/go/chasquid/internal/config"
 	"blitiri.com.ar/go/chasquid/internal/expvarom"
+	"blitiri.com.ar/go/chasquid/internal/nettrace"
 	"blitiri.com.ar/go/log"
 	"google.golang.org/protobuf/encoding/prototext"
 
@@ -19,6 +23,70 @@ import (
 	_ "net/http/pprof"
 )
 
+// Build information, overridden at build time using
+// -ldflags="-X main.version=blah".
+var (
+	version      = ""
+	sourceDateTs = ""
+)
+
+var (
+	versionVar = expvar.NewString("chasquid/version")
+
+	sourceDate      time.Time
+	sourceDateVar   = expvar.NewString("chasquid/sourceDateStr")
+	sourceDateTsVar = expvarom.NewInt("chasquid/sourceDateTimestamp",
+		"timestamp when the binary was built, in seconds since epoch")
+)
+
+func parseVersionInfo() {
+	bi, ok := debug.ReadBuildInfo()
+	if !ok {
+		panic("unable to read build info")
+	}
+
+	dirty := false
+	gitRev := ""
+	gitTime := ""
+	for _, s := range bi.Settings {
+		switch s.Key {
+		case "vcs.modified":
+			if s.Value == "true" {
+				dirty = true
+			}
+		case "vcs.time":
+			gitTime = s.Value
+		case "vcs.revision":
+			gitRev = s.Value
+		}
+	}
+
+	if sourceDateTs != "" {
+		sdts, err := strconv.ParseInt(sourceDateTs, 10, 0)
+		if err != nil {
+			panic(err)
+		}
+
+		sourceDate = time.Unix(sdts, 0)
+	} else {
+		sourceDate, _ = time.Parse(time.RFC3339, gitTime)
+	}
+	sourceDateVar.Set(sourceDate.Format("2006-01-02 15:04:05 -0700"))
+	sourceDateTsVar.Set(sourceDate.Unix())
+
+	if version == "" {
+		version = sourceDate.Format("20060102")
+
+		if gitRev != "" {
+			version += fmt.Sprintf("-%.9s", gitRev)
+		}
+		if dirty {
+			version += "-dirty"
+		}
+	}
+	versionVar.Set(version)
+}
+
 func launchMonitoringServer(conf *config.Config) {
 	log.Infof("Monitoring HTTP server listening on %s", conf.MonitoringAddress)
 
@@ -56,6 +124,7 @@ func launchMonitoringServer(conf *config.Config) {
 	http.HandleFunc("/metrics", expvarom.MetricsHandler)
 	http.HandleFunc("/debug/flags", debugFlagsHandler)
 	http.HandleFunc("/debug/config", debugConfigHandler(conf))
+	http.HandleFunc("/debug/traces", nettrace.RenderTraces)
 
 	if err := srv.ListenAndServe(); err != http.ErrServerClosed {
 		log.Fatalf("Monitoring server failed: %v", err)
@@ -82,6 +151,13 @@ var monitoringHTMLIndex = template.Must(
   body {
     font-family: sans-serif;
   }
+  @media (prefers-color-scheme: dark) {
+    body {
+      background: #121212;
+      color: #c9d1d9;
+    }
+    a { color: #44b4ec; }
+  }
 </style>
 </head>
 
@@ -104,8 +180,7 @@ os hostname <i>{{.Hostname}}</i><br>
   <li><a href="/debug/queue">queue</a>
   <li>monitoring
     <ul>
-      <li><a href="/debug/requests?exp=1">requests (short-lived)</a>
-      <li><a href="/debug/events?exp=1">events (long-lived)</a>
+      <li><a href="/debug/traces">traces</a>
 	  <li><a href="https://blitiri.com.ar/p/chasquid/monitoring/#variables">
 	        exported variables</a>:
           <a href="/debug/vars">expvar</a>
