@@ -1,7 +1,7 @@
 #!/bin/bash
 
 set -e
-. $(dirname ${0})/../util/lib.sh
+. "$(dirname "$0")/../util/lib.sh"
 
 init
 check_hostaliases
@@ -22,7 +22,7 @@ rm -f .mail/user@testserver
 
 
 # Test that we get mail back for a failed delivery
-run_msmtp fail@testserver < content
+smtpc fail@testserver < content
 wait_for_file .mail/user@testserver
 mail_diff expected_dsr .mail/user@testserver
 
